/*
 * Basic example of using the softdevice App timer
 *
 * Requires S132 Softdevice - S132_nRF52_2.0.0 (Id: 0x0081)
 *
 * LED 1 Flshaes on briefly using a single shot timer that is releaded after each flash
 * LED 2 toggles based on a repeating timer at IO_TOGGLE_FREQUENCY
 * RTT Debugging is enabled by attaching the SEGGER RTT Terminal application to the target board
 *
 *
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include <string.h>
#include "app_error.h"
#include "nrf_gpio.h"
#include "app_scheduler.h"
#include "softdevice_handler.h"
#include "app_timer_appsh.h"
#include "nordic_common.h"
#include "fds.h"
#include "fstorage.h"
#include "SEGGER_RTT.h"




//this macro let's us use standard printf() functionality with SEGGER RTT
#define RTT_PRINTF(...)\
do{\
   char str[64];\
	sprintf(str, __VA_ARGS__);\
	SEGGER_RTT_WriteString(0, str);\
}while(0)


#define LED1_PIN    							 17    //PO.17 for PCA10040 LED1 
#define LED2_PIN                   18    ///PO.18 for PCA10040 LED2



// Low frequency clock source to be used by the SoftDevice
#define NRF_CLOCK_LFCLKSRC      {.source        = NRF_CLOCK_LF_SRC_XTAL,            \
                                 .rc_ctiv       = 0,                                \
                                 .rc_temp_ctiv  = 0,                                \
                                 .xtal_accuracy = NRF_CLOCK_LF_XTAL_ACCURACY_20_PPM}



#define APP_TIMER_PRESCALER             0                                                                                
#define APP_TIMER_OP_QUEUE_SIZE         4                                          
#define LED_BLINK_OFF_TIME							APP_TIMER_TICKS(2000, APP_TIMER_PRESCALER)  
#define LED_BLINK_ON_TIME							  APP_TIMER_TICKS(2000, APP_TIMER_PRESCALER) 
#define IO_TOGGLE_FREQUENCY			        APP_TIMER_TICKS(1000, APP_TIMER_PRESCALER)
#define SCHED_MAX_EVENT_DATA_SIZE       sizeof(app_timer_event_t)                 
#define SCHED_QUEUE_SIZE                10            

APP_TIMER_DEF(m_led_blink_timer_id);
APP_TIMER_DEF(m_io_pin_toggle_timer_id);

static uint8_t     											LED_state;
static volatile uint8_t write_flag=0;	
uint8_t somevar;






static void io_pin_toggle_timer_handler(void *p_context)
{
	nrf_gpio_pin_toggle(LED2_PIN);
	
	
	
}




static void led_blink_timer_handler(void * p_context)
{
    uint32_t err_code, next_timer_interval; 

		if(LED_state)
		{
			 nrf_gpio_pin_clear(LED1_PIN);      		//turn On the LED
			 next_timer_interval = LED_BLINK_ON_TIME;   //stay On for ON_TIME
		}
		else
		{
			 nrf_gpio_pin_set(LED1_PIN);  					//turn off the LED
			 next_timer_interval = LED_BLINK_OFF_TIME;  //stay off for OFF_TIME 
		}
		LED_state = !LED_state;	
		
	  err_code = app_timer_start(m_led_blink_timer_id, next_timer_interval, NULL);  //reload the timer
    APP_ERROR_CHECK(err_code);
		
}





static void gpio_init(void)
{

	  nrf_gpio_cfg_output(LED1_PIN);
	  nrf_gpio_cfg_output(LED2_PIN); 
	  nrf_gpio_pin_set(LED1_PIN);           //set Pin High at bootup
	  nrf_gpio_pin_set(LED2_PIN);           //set Pin High at bootup
}



static void timers_create(void)
{
    uint32_t err_code;  
	  
    APP_TIMER_APPSH_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, true);

	  LED_state = 0;
	
    err_code = app_timer_create(&m_led_blink_timer_id, APP_TIMER_MODE_SINGLE_SHOT, led_blink_timer_handler);
    APP_ERROR_CHECK(err_code);
	  
	  err_code = app_timer_create(&m_io_pin_toggle_timer_id, APP_TIMER_MODE_REPEATED, io_pin_toggle_timer_handler);
    APP_ERROR_CHECK(err_code);
}



static void timers_start(void)
{
    uint32_t err_code;
		
    err_code = app_timer_start(m_led_blink_timer_id, LED_BLINK_OFF_TIME, NULL);
    APP_ERROR_CHECK(err_code); 
	
	  err_code = app_timer_start(m_io_pin_toggle_timer_id, IO_TOGGLE_FREQUENCY, NULL);
    APP_ERROR_CHECK(err_code); 
}


static void scheduler_init(void)
{
    APP_SCHED_INIT(SCHED_MAX_EVENT_DATA_SIZE, SCHED_QUEUE_SIZE);
}


static void soft_device_init(void)
{
	 nrf_clock_lf_cfg_t clock_lf_cfg = NRF_CLOCK_LFCLKSRC;
    
    // Initialize the SoftDevice handler module.
    SOFTDEVICE_HANDLER_INIT(&clock_lf_cfg, NULL);
}



static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}




static void my_fds_evt_handler(fds_evt_t const * const p_fds_evt)
{
    switch (p_fds_evt->id)
    {
        case FDS_EVT_INIT:
            if (p_fds_evt->result != FDS_SUCCESS)
            {
                // Initialization failed.
            }
            break;
				case FDS_EVT_WRITE:
						if (p_fds_evt->result == FDS_SUCCESS)
						{
							write_flag=1;
						}
						break;
        default:
            break;
    }
}
static ret_code_t fds_test_write(void)
{
		#define FILE_ID     0x1111
		#define REC_KEY     0x2222
		static uint32_t const m_deadbeef[2] = {0xDEADBEEF,0xBAADF00D};
		fds_record_t        record;
		fds_record_desc_t   record_desc;
		fds_record_chunk_t  record_chunk;
		// Set up data.
		record_chunk.p_data         = m_deadbeef;
		record_chunk.length_words   = 2;
		// Set up record.
		record.file_id              = FILE_ID;
		record.key              		= REC_KEY;
		record.data.p_chunks       = &record_chunk;
		record.data.num_chunks   = 1;
				
		ret_code_t ret = fds_record_write(&record_desc, &record);
		if (ret != FDS_SUCCESS)
		{
				return ret;
		}
		 RTT_PRINTF("Writing Record ID = %d \n",record_desc.record_id);
		return NRF_SUCCESS;
}

static ret_code_t fds_read(void)
{
		#define FILE_ID     0x1111
		#define REC_KEY     0x2222
		fds_flash_record_t  flash_record;
		fds_record_desc_t   record_desc;
		fds_find_token_t    ftok ={0};//Important, make sure you zero init the ftok token
		uint32_t *data;
		uint32_t err_code;
		
		RTT_PRINTF("Start searching... \n");
		// Loop until all records with the given key and file ID have been found.
		while (fds_record_find(FILE_ID, REC_KEY, &record_desc, &ftok) == FDS_SUCCESS)
		{
				err_code = fds_record_open(&record_desc, &flash_record);
				if ( err_code != FDS_SUCCESS)
				{
					return err_code;		
				}
				
				RTT_PRINTF("Found Record ID = %d\n",record_desc.record_id);
				RTT_PRINTF("Data = ");
				data = (uint32_t *) flash_record.p_data;
				for (uint8_t i=0;i<flash_record.p_header->tl.length_words;i++)
				{
					RTT_PRINTF("0x%8x ",data[i]);
				}
				RTT_PRINTF("\n");
				// Access the record through the flash_record structure.
				// Close the record when done.
				err_code = fds_record_close(&record_desc);
				if (err_code != FDS_SUCCESS)
				{
					return err_code;	
				}
		}
		return NRF_SUCCESS;
		
}

static ret_code_t fds_test_find_and_delete (void)
{
	#define FILE_ID     0x1111
		#define REC_KEY     0x2222
		fds_record_desc_t   record_desc;
		fds_find_token_t    ftok;
	
		ftok.page=0;
		ftok.p_addr=NULL;
		// Loop and find records with same ID and rec key and mark them as deleted. 
		while (fds_record_find(FILE_ID, REC_KEY, &record_desc, &ftok) == FDS_SUCCESS)
		{
			fds_record_delete(&record_desc);
			RTT_PRINTF("Deleted record ID: %d \n",record_desc.record_id);
		}
		// call the garbage collector to empty them, don't need to do this all the time, this is just for demonstration
		ret_code_t ret = fds_gc();
		if (ret != FDS_SUCCESS)
		{
				return ret;
		}
		return NRF_SUCCESS;
}

static ret_code_t fds_test_init (void)
{
	
		ret_code_t ret = fds_register(my_fds_evt_handler);
		if (ret != FDS_SUCCESS)
		{
					return ret;
				
		}
		ret = fds_init();
		if (ret != FDS_SUCCESS)
		{
				return ret;
		}
		
		return NRF_SUCCESS;
		
}




int main(void)
{
    uint32_t err_code;
  
  	RTT_PRINTF("I/O Test nRF52 s132 v2.0.0 Nordic SDK v11.0.0\n"); 
 	  gpio_init();
    timers_create();
	  soft_device_init();
	  scheduler_init();

	  LED_state = 0;
    err_code =fds_test_init();
		APP_ERROR_CHECK(err_code);
		err_code = fds_test_find_and_delete();
		APP_ERROR_CHECK(err_code);
		err_code =fds_test_write();
		APP_ERROR_CHECK(err_code);
		//wait until the write is finished. 
		while (write_flag==0);
		fds_read();
   
    timers_start();
  
  for (;;)
    {
        app_sched_execute();
        power_manage();
    }
}


